/* This file contains the kernels i.e. the functions to be executed on the GPU */

#include <stdio.h>
#include <stdlib.h>
#include "kernels.h"
#include "common.h"

__device__
void add_two_device(CUdeviceptr number)
{
    (* (int*) number)+=2;
}

__global__
void add_three_global(CUdeviceptr number)
{
    (* (int*) number)++;
    add_two_device(number);
}

__global__
void add_one_global(CUdeviceptr number)
{
    (* (int*) number)++;
}

void first_test(CUdeviceptr dptr, void* va)
{
    int set, get;
    printf("Use the nvidia api\n");
    set = 4242;
    get = 0;
    printf("set = %d\nget = %d\n",set,get);
    assert_cu( cuMemcpyHtoD(dptr,&set,sizeof(set)) );
    add_three_global<<< 1,1 >>>(dptr);
    cudaDeviceSynchronize();
    assert_cu( cuMemcpyDtoH(&get,dptr,sizeof(get)) );
    printf("set = %d\nget = %d\n",set,get);

    printf("Use the gdr api\n");
    set = 4242;
    get = 0;
    printf("set = %d\nget = %d\nva = %d\n",set,get,*(int*) va);
    assert_gdr( gdr_copy_to_bar(va,&set,sizeof(set)) );
    add_one_global<<< 1,1 >>>(dptr);
    cudaDeviceSynchronize();
    assert_gdr( gdr_copy_from_bar(&get,va,sizeof(get)) );
    printf("set = %d\nget = %d\nva = %d\n",set,get,*(int*) va);
}
